from .models import Subscription
from django.http.response import HttpResponseRedirect
from django.contrib import messages
from django.utils.functional import cached_property
from .utils import render_template

class SubscriptionMixin:
    def get(self, request, *args, **kwargs):
        if "subscribe" in request.GET:
            return self.subscribe()
        elif "unsubscribe" in request.GET:
            return self.unsubscribe()
        
        return super().get(request, *args, **kwargs)
    
    def subscribe(self):
        subscription = self.get_subscription()
        if subscription and not subscription.pk:
            subscription.save()
            title = render_template(self.subscription_trigger, "title.djtxt")
            messages.add_message(self.request, messages.INFO, 
                                 f"Successfully subscribed for {title}.")
        return HttpResponseRedirect(self.request.path)
    
    def unsubscribe(self):
        subscription = self.get_subscription()
        if subscription and subscription.pk:
            subscription.delete()
            title = render_template(self.subscription_trigger, "title.djtxt")
            messages.add_message(self.request, messages.INFO, 
                                 f"Successfully unsubscribed {title}.")
        return HttpResponseRedirect(self.request.path)
    
    def get_subscription(self):
        if self.subscription_trigger:
            return Subscription.objects.get_or_prepare(
                user=self.request.user, 
                content_object=self.subscription_trigger)
        else:
            return None
    
    @cached_property
    def subscription_trigger(self):
        return self.get_subscription_trigger()
    
    def get_subscription_trigger(self):
        raise NotImplementedError()
    
    def get_context_data(self, **kwargs):
        return {
                **super().get_context_data(**kwargs),
                'subscription' : self.get_subscription()
            }





import re
import logging
from django.db import models
from django.conf import settings 
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.dispatch.dispatcher import receiver
from django.db.models.signals import post_save
from django.core.mail import send_mail
from .utils import render_template

logger=logging.getLogger(__name__)

# Create your models here.
class SubscriptionQuerySet(models.QuerySet):
    
    def filter(self, *args, **kwargs):
        content_object=kwargs.pop('content_object', None)
        if content_object:
            kwargs['content_type']=ContentType.objects.get_for_model(content_object._meta.model)
            try:
                kwargs['object_id']=int(content_object.pk)
            except ValueError:
                return self.none()
        return super().filter(*args, **kwargs)
    
    def get_or_prepare(self, user, content_object):
        try:
            return self.get(user=user, content_object=content_object)
        except self.model.DoesNotExist:
            return self.model(user=user, content_object=content_object)

class Subscription(models.Model):
    user = models.ForeignKey("auth.User", on_delete=models.CASCADE)
    
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey('content_type', 'object_id')
    
    class Meta:
        unique_together=('user', 'content_type', 'object_id')

    objects = SubscriptionQuerySet.as_manager()
    
    def __str__(self):
        return f"{self.user} :-> {self.content_object}"
    
@receiver(post_save)
def notify_subscribers(instance, **kwargs):
    recipients = list(Subscription.objects.filter(
        content_object=instance).values_list("user__email", flat=True))
    if not recipients:
        return
    try:
        send_mail(subject = re.sub(r"\s+", " ", render_template(instance, "title.djtxt")).strip(), 
                  message = render_template(instance, "message.djtxt"), 
                  from_email=settings.DEFAULT_FROM_EMAIL, 
                  recipient_list=recipients, 
                  html_message = render_template(instance, "message.djhtml"),
                  fail_silently=True)
    except Exception:
        logger.exception(f"Exception raised when sending {instance} update notification")



# Generated by Django 3.1 on 2020-08-28 09:48

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Subscription',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField()),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'unique_together': {('user', 'content_type', 'object_id')},
            },
        ),
    ]



from django.template.loader import render_to_string
from django.conf import settings

def render_template(instance, name):
    return render_to_string(
        f"{instance._meta.app_label}/notifications/{instance._meta.object_name.lower()}/{name}",
        {
            "object" : instance,
            'site_url' : settings.REQUEST_BASE_URL
        })


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    author='Kamil Mierzejewski',
    name='django-update-notifications',
    version='0.1.4',
    description='A framework for sending e-mail notifications about model updates.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=setuptools.find_packages(),
    install_requires=['django>=3.0.8'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.7',
    ],
)



from django.contrib import admin

from .models import Subscription

@admin.register(Subscription)
class SubscriptionAdmin(admin.ModelAdmin):
    list_display=('content_object', 'user')
    list_filter=('user',)
